=begin
v2.6 point component
Copyright 2009, Atelier

 THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
  Points a selected component along its z (blue) axis to a selected target on the model.
  The component pivots on its origin.
  
  When the "IDX ceiling light #01" is selected, the user will be given the option to point 
  the light without rotating the fixture. When rendered by IDX Renditioner, this fixture's light 
  beam will point at the target while the  component remains in place, in the manner of a 
  recessed adjustable spolight. Current conditions:
  1. Adjustable light fixture can not be readjusted.
  2. The light fixture must be a unique component. If you insert another instance of a adjusted
  component, it's light cast will already be adjusted.
  3. The light fixture to be adjusted must be on a horizontal ceiling.
=end

require 'sketchup'

unless file_loaded? File::basename(__FILE__)
	UI.add_context_menu_handler do |menu|							# add menu to r-click selection menu
		selection=Sketchup.active_model.selection
		if selection[0].is_a? Sketchup::ComponentInstance #if selection
			idx_fix=" "
			menu.add_separator
			sel_entities=selection[0].definition.entities
			sel_entities.each do |e|
				if e.is_a? Sketchup::ComponentInstance and e.definition.name=="LightDisc" and idx_fix==" "
					if selection[0].transformation.zaxis.z==-1
						sub_menu=menu.add_item("Point light fixture's beam") { hdt_lighttool_idx() }
						idx_fix="found"
					else
						menu.add_submenu("Point light requires vertical alignment")
						idx_fix="found"
					end
					else if e.is_a? Sketchup::ComponentInstance and e.definition.name.split("#").first=="LightDisc" and idx_fix==" "#use later
						menu.add_submenu("Point light <UNAVAILABLE>")
						idx_fix="found"
					end
				end
			end
			sub_menu=menu.add_item("Point component's z.axis") { hdt_pointtool_idx() }
		end
	end
	file_loaded File::basename(__FILE__)
end

class Atelier_pointtool_new
#result = Sketchup.send_action "showRubyPanel:"
	
	def initialize(light_tool)
		@sel = light_tool									#flag
	end

	def activate
		model=Sketchup.active_model
		if Sketchup.version.to_i<7
			model.start_operation "Draw points"
		else
			model.start_operation "Draw points",true
		end
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@drawn=false
		if Sketchup.active_model.rendering_options["HideConstructionGeometry"]==true 
			Sketchup.active_model.rendering_options["HideConstructionGeometry"]=false 
		end
		self.reset(nil)
	end

	def reset(view)
		Sketchup::set_status_text("Select blue axis target:")
		@ip.clear
		@ip1.clear
		if view
			view.tooltip=nil
			view.invalidate if @drawn
		end
		@drawn = false
	end

	def onMouseMove(flags, x, y, view)
		@ip.pick view, x, y
		if @ip != @ip1
			view.invalidate if @ip.display? or @ip1.display?
			@ip1.copy! @ip
			view.tooltip = @ip1.tooltip
		end
		view.tooltip = "Select blue axis target:"
	end

	def onLButtonDown(flags, x, y, view)
		@ip1.pick view, x, y
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
		self.create_geometry(@ip1.position,view) 		#if self.no_points_here?(@ip1.position)
	end

	def draw(view)
		if @ip1.display?
			@ip1.draw(view)
			@drawn = true
		end
	end

	def create_geometry(p,view)
		view.model.active_entities.add_cpoint p
		point_to_component(p)
		Sketchup.send_action "selectSelectionTool:"
		view.invalidate
	end

	def point_to_component(point_here)
#		result = Sketchup.send_action "showRubyPanel:"		
		model = Sketchup.active_model
		entities = model.entities
		selection = model.selection

		fixture=selection[0]										#get apporx vector from fixture to target
		#cl=model.active_entities.add_cline(fixture.transformation.origin, point_here)	#draw line
		if @sel == "yes" 												#come here to rotate sub component "LightDisc"
			if fixture.definition.count_instances==1		#must be parent instance, add instance for make_unique so that parent is unchanged.
				key_fixture=entities.add_instance(fixture.definition,fixture.transformation)
				key_flag="erase"										#flag to erase key instance below				
			else
				key_flag="keep"
			end
			fixture.make_unique									#create a new component definition
			#point_vector=((fixture.transformation.origin.vector_to point_here).normalize!)
			fixture_entities=fixture.definition.entities	#create array of its entities###
			flag=" "														#found flag, cleared
			fixture_entities.each do |disc|						#search for adjustable disc, we expect to find because test for it already done at top of program
				if disc.is_a? Sketchup::ComponentInstance and disc.definition.name=="LightDisc" and flag==" "
					@light_disc=disc								#save component id
					flag="found"										#set flag
				end
			end
			@light_disc.make_unique							#make new component definition
			light_disc_origin=[@light_disc.transformation.origin.x+0.25,@light_disc.transformation.origin.y+0.25,@light_disc.transformation.origin.z]
			point_vector=((light_disc_origin.vector_to [point_here.x-fixture.transformation.origin.x, point_here.y-fixture.transformation.origin.y, point_here.z-fixture.transformation.origin.z]).normalize!)
			@light_disc.transform! Geom::Transformation.new(light_disc_origin,@light_disc.transformation.xaxis, Math::PI)	#flip disc over on origin
			old_trans=@light_disc.transformation					#save current transformation of light disc		
			new_transformation = Geom::Transformation.new([0,0,0],point_vector)	#get vector transformation for pointing disc to target
			@light_disc.transformation = new_transformation	#transform disc (point to target)
																							#rotate ent's back to inverse of new_transformation
			@light_disc.definition.entities.transform_entities(new_transformation.invert!,@light_disc.definition.entities.to_a)
			@light_disc.transform! old_trans							#return disc to its original location
			if key_flag=="erase"												#erase key instance?
				entities.erase_entities key_fixture						#erase original parent instance
			end
		else																				#come here to rotate whole component
			point_vector=((fixture.transformation.origin.vector_to point_here).normalize!)
			fixture.transformation=Geom::Transformation.new(fixture.transformation.origin,point_vector)
			fixture.transform! Geom::Transformation.new(fixture.transformation.origin,fixture.transformation.zaxis, (Math::PI*3)/2)
		end
	end
end

def hdt_pointtool_idx()														#point component
	light_tool="none"
	Sketchup.active_model.select_tool Atelier_pointtool_new.new(light_tool)
end

def hdt_lighttool_idx()														#point component
	light_tool="yes"
	Sketchup.active_model.select_tool Atelier_pointtool_new.new(light_tool)
end

